C ========================================================
C  ATPDesigner - Design and Simulation of Power Networks
C  Version 2.00.25 - 29.10.2008
C  Copyright  2005-08
C  29.10.2008 , 20:33:27
C ========================================================
C ATPDESIGNER START BLOCK MODELS
MODELS -- Start of the MODELS Section

  -- ATPDESIGNER START MODEL 012
  -- 012 - Transformation (ABC -> 012)
  MODEL SYMM

    -- Input Interface at MODEL level : MODELS -> MODEL source code
    INPUT API  -- Current Phase IA
    INPUT BPI  -- Current Phase IB
    INPUT CPI  -- Current Phase IC
    INPUT APV  -- Phase-to-Ground Voltage VAG
    INPUT BPV  -- Phase-to-Ground Voltage CPVG
    INPUT CPV  -- Phase-to-Ground Voltage VCG

    CONST PowerNetFreq {val: 50}  -- POWER FREQUENCY [Hz]
    CONST PowerNetCycle {val: 0.02}  -- Cycle [s]
    CONST SamplingFreq {val: 1000}  -- Sampling Frequency [Hz]
    CONST SamplingDT {val: 0.001}  -- Sampling Rate [s]
    CONST SamplesPerCycle {val: 20}  -- Samples per Cycle

    DATA MeasLocation {dflt: -1.0} -- ID of the measuring location

    CONST RadToDeg {val: 180.0/pi}
	
	-- (a + jb) x (c + jd)
	FUNCTION CMultRe(a, b, c, d) := a*c - b*d
	FUNCTION CMultIm(a, b, c, d) := b*c + a*d

    -- Declaration of all variables
    VAR

      -- Sampled 012-voltages of the last cycle
      APVBuffer[0..(SamplesPerCycle - 1)]
      BPVBuffer[0..(SamplesPerCycle - 1)]
      CPVBuffer[0..(SamplesPerCycle - 1)]

      -- Sampled 012-currents of the last cycle
      APIBuffer[0..(SamplesPerCycle - 1)]
      BPIBuffer[0..(SamplesPerCycle - 1)]
      CPIBuffer[0..(SamplesPerCycle - 1)]

      -- Filter coefficients (Discrete Fourier Analysis)
      CosKoeff[0..(SamplesPerCycle - 1)]
      SinKoeff[0..(SamplesPerCycle - 1)]

      -- Re=Real part, Im=Imaginary part
      -- Amount in V or A, Phase in Degree
      APVRe, APVIm
      BPVRe, BPVIm
      CPVRe, CPVIm
      APIRe, APIIm
      BPIRe, BPIIm
      CPIRe, CPIIm
      APVAmount, APVPhase
      BPVAmount, BPVPhase
      CPVAmount, CPVPhase
      APIAmount, APIPhase
      BPIAmount, BPIPhase
      CPIAmount, CPIPhase
      Omega -- Angular Frequency in Hz

      -- Voltages and currents 012-system
      -- V=Voltage, I=Current
      V0Re, V0Im
      V1Re, V1Im
      V2Re, V2Im
      I0Re, I0Im
      I1Re, I1Im
      I2Re, I2Im
	  V0Amount, I0Amount
	  V1Amount, I1Amount
	  V2Amount, I2Amount
	  V0Phase, I0Phase
	  V1Phase, I1Phase
	  V2Phase, I2Phase

      -- Coefficients of the 012 - transformation ABC -> 012
      -- |APV|   |TR1[1] TR1[2] TR1[3]|   |VAG|
      -- |BPV| = |TR2[1] TR2[2] TR2[3]| x |VBG|
      -- |CPV|   |TR3[1] TR3[2] TR3[3]|   |VCG|
      -- Real part
      TR1[1..3]  -- Line 1
      TR2[1..3]  -- Line 2
      TR3[1..3]  -- Line 3
      -- Imaginary part
      TI1[1..3]  -- Line 1
      TI2[1..3]  -- Line 2
      TI3[1..3]  -- Line 3

    -- One-time initialization of variables, etc. at simulation start
    INIT

      -- Coefficients of the 012 - transformation ABC -> 012
      -- for transformation of phase-to-ground voltages
      TR1[1] := 1/3
      TR1[2] := 1/3
      TR1[3] := 1/3
      TR2[1] := 1/3
      TR2[2] := -1/6
      TR2[3] := -1/6
      TR3[1] := 1/3
      TR3[2] := -1/6
      TR3[3] := -1/6

      -- Coefficients of the 012 - transformation ABC -> 012
      -- for transformation of phase currents
      TI1[1] := 0
      TI1[2] := 0
      TI1[3] := 0
      TI2[1] := 0
      TI2[2] := sqrt(3) / 6
      TI2[3] := -1 * sqrt(3) / 6
      TI3[1] := 0
      TI3[2] := -1 * sqrt(3) / 6
      TI3[3] := sqrt(3) / 6

      V0Re := 0.0
      V1Re := 0.0
      V2Re := 0.0
      I0Re := 0.0
      I1Re := 0.0
      I2Re := 0.0
      V0Im := 0.0
      V1Im := 0.0
      V2Im := 0.0
      I0Im := 0.0
      I1Im := 0.0
      I2Im := 0.0
	  V0Amount := 0.0
	  V1Amount := 0.0
	  V2Amount := 0.0
	  I0Amount := 0.0
	  I1Amount := 0.0
	  I2Amount := 0.0
	  V0Phase := 0.0
	  V1Phase := 0.0
	  V2Phase := 0.0
	  I0Phase := 0.0
	  I1Phase := 0.0
	  I2Phase := 0.0

      for i := 0 to (SamplesPerCycle - 1) do
        APVBuffer[i] := 0.0
        BPVBuffer[i] := 0.0
        CPVBuffer[i] := 0.0
        APIBuffer[i] := 0.0
        BPIBuffer[i] := 0.0
        CPIBuffer[i] := 0.0
      endfor

      APVRe := 0.0
      APVIm := 0.0
      BPVRe := 0.0
      BPVIm := 0.0
      CPVRe := 0.0
      CPVIm := 0.0
      APIRe := 0.0
      APIIm := 0.0
      BPIRe := 0.0
      BPIIm := 0.0
      CPIRe := 0.0
      CPIIm := 0.0
      APVAmount := 0.0
      BPVPhase := 0.0
      CPVAmount := 0.0
      APIAmount := 0.0
      BPIPhase := 0.0
      CPIAmount := 0.0
      APVPhase := 0.0
      BPVAmount := 0.0
      CPVPhase := 0.0
      APIPhase := 0.0
      BPIAmount := 0.0
      CPIPhase := 0.0
      Omega := 2.0 * pi * PowerNetFreq

      -- Calculation of DFT Filter Coefficients
      for i := 0 to (SamplesPerCycle - 1) do
        CosKoeff[i] := cos(Omega * SamplingDT * i)
        SinKoeff[i] := sin(Omega * SamplingDT * i)
      endfor

    ENDINIT

    -- Execution at each simulation time step
    EXEC

      -- Shifting the sampled values one time step to the past
      for i := 0 to (SamplesPerCycle - 2) do
        APVBuffer[i] := APVBuffer[i+1]
        BPVBuffer[i] := BPVBuffer[i+1]
        CPVBuffer[i] := CPVBuffer[i+1]
        APIBuffer[i] := APIBuffer[i+1]
        BPIBuffer[i] := BPIBuffer[i+1]
        CPIBuffer[i] := CPIBuffer[i+1]
      endfor

      -- Insert the new sampled value
      APVBuffer[SamplesPerCycle - 1] := APV
      BPVBuffer[SamplesPerCycle - 1] := BPV
      CPVBuffer[SamplesPerCycle - 1] := CPV
      APIBuffer[SamplesPerCycle - 1] := API
      BPIBuffer[SamplesPerCycle - 1] := BPI
      CPIBuffer[SamplesPerCycle - 1] := CPI

      -- Reset variables
      APVRe := 0.0
      APVIm := 0.0
      BPVRe := 0.0
      BPVIm := 0.0
      CPVRe := 0.0
      CPVIm := 0.0
      APIRe := 0.0
      APIIm := 0.0
      BPIRe := 0.0
      BPIIm := 0.0
      CPIRe := 0.0
      CPIIm := 0.0
      APVAmount := 0.0
      BPVPhase := 0.0
      CPVAmount := 0.0
      APIAmount := 0.0
      BPIPhase := 0.0
      CPIAmount := 0.0
      APVPhase := 0.0
      BPVAmount := 0.0
      CPVPhase := 0.0
      APIPhase := 0.0
      BPIAmount := 0.0
      CPIPhase := 0.0

      -- Processing the DFT for the last cycle
      for i := 0 to (SamplesPerCycle - 1) do
        APVRe := APVRe + APVBuffer[i] * SinKoeff[i]
        APVIm := APVIm + APVBuffer[i] * CosKoeff[i]
        BPVRe := BPVRe + BPVBuffer[i] * SinKoeff[i]
        BPVIm := BPVIm + BPVBuffer[i] * CosKoeff[i]
        CPVRe := CPVRe + CPVBuffer[i] * SinKoeff[i]
        CPVIm := CPVIm + CPVBuffer[i] * CosKoeff[i]
        APIRe := APIRe + APIBuffer[i] * SinKoeff[i]
        APIIm := APIIm + APIBuffer[i] * CosKoeff[i]
        BPIRe := BPIRe + BPIBuffer[i] * SinKoeff[i]
        BPIIm := BPIIm + BPIBuffer[i] * CosKoeff[i]
        CPIRe := CPIRe + CPIBuffer[i] * SinKoeff[i]
        CPIIm := CPIIm + CPIBuffer[i] * CosKoeff[i]
      endfor

      APVRe := APVRe * 2.0 / SamplesPerCycle
      APVIm := APVIm * 2.0 / SamplesPerCycle
      BPVRe := BPVRe * 2.0 / SamplesPerCycle
      BPVIm := BPVIm * 2.0 / SamplesPerCycle
      CPVRe := CPVRe * 2.0 / SamplesPerCycle
      CPVIm := CPVIm * 2.0 / SamplesPerCycle
      APIRe := APIRe * 2.0 / SamplesPerCycle
      APIIm := APIIm * 2.0 / SamplesPerCycle
      BPIRe := BPIRe * 2.0 / SamplesPerCycle
      BPIIm := BPIIm * 2.0 / SamplesPerCycle
      CPIRe := CPIRe * 2.0 / SamplesPerCycle
      CPIIm := CPIIm * 2.0 / SamplesPerCycle

      -- Peak BPVlue of the fundamental frequency
      APVAmount := sqrt((APVRe * APVRe) + (APVIm * APVIm))
      BPVAmount := sqrt((BPVRe * BPVRe) + (BPVIm * BPVIm))
      CPVAmount := sqrt((CPVRe * CPVRe) + (CPVIm * CPVIm))
      APIAmount := sqrt((APIRe * APIRe) + (APIIm * APIIm))
      BPIAmount := sqrt((BPIRe * BPIRe) + (BPIIm * BPIIm))
      CPIAmount := sqrt((CPIRe * CPIRe) + (CPIIm * CPIIm))

      -- Absolute phase angle of the fundamental frequency
      if (APVIm <> 0.0) and (APVRe <> 0.0) then
        APVPhase := atan2(APVIm, APVRe) * RadToDeg
      endif

      if (BPVIm <> 0.0) and (BPVRe <> 0.0) then
        BPVPhase := atan2(BPVIm, BPVRe) * RadToDeg
      endif

      if (CPVIm <> 0.0) and (CPVRe <> 0.0) then
        CPVPhase := atan2(CPVIm, CPVRe) * RadToDeg
      endif

      if (APIIm <> 0.0) and (APIRe <> 0.0) then
        APIPhase := atan2(APIIm, APIRe) * RadToDeg
      endif

      if (BPIIm <> 0.0) and (BPIRe <> 0.0) then
        BPIPhase := atan2(BPIIm, BPIRe) * RadToDeg
      endif

      if (CPIIm <> 0.0) and (CPIRe <> 0.0) then
        CPIPhase := atan2(CPIIm, CPIRe) * RadToDeg
      endif
	  
	  -- zero-sequence system voltage V0	  
	  V0Re := CMultRe(TR1[1], TI1[1], APVRe, APVIm)
	  V0Re := V0Re + CMultRe(TR1[2], TI1[2], BPVRe, BPVIm)
	  V0Re := V0Re + CMultRe(TR1[3], TI1[3], CPVRe, CPVIm)

	  V0Im := CMultIm(TR1[1], TI1[1], APVRe, APVIm)
	  V0Im := V0Im + CMultIm(TR1[2], TI1[2], BPVRe, BPVIm)
	  V0Im := V0Im + CMultIm(TR1[3], TI1[3], CPVRe, CPVIm)

	  V0Amount := sqrt((V0Re * V0Re) + (V0Im * V0Im))
      if (V0Im <> 0.0) and (V0Re <> 0.0) then
        V0Phase := atan2(V0Im, V0Re) * RadToDeg
      endif
	    
      -- write ("-- ++++++++++++++++++++++++++++++++++++++++++++++++++")
	  -- write ("t=", t)
	  -- write ("APVRe=", APVRe, "  APVIm=", APVIm)
	  -- write ("BPVRe=", BPVRe, "  BPVIm=", BPVIm)
	  -- write ("CPVRe=", CPVRe, "  CPVIm=", CPVIm)
	  -- write ("APVB=", APVAmount, "  APVP=", APVPhase)
	  -- write ("BPVB=", BPVAmount, "  BPVP=", BPVPhase)
	  -- write ("CPVB=", CPVAmount, "  CPVP=", CPVPhase)

		-- positive-sequence system voltage V1	  
	  V1Re := CMultRe(TR2[1], TI2[1], APVRe, APVIm)
	  V1Re := V1Re + CMultRe(TR2[2], TI2[2], BPVRe, BPVIm)
	  V1Re := V1Re + CMultRe(TR2[3], TI2[3], CPVRe, CPVIm)

	  V1Im := CMultIm(TR2[1], TI2[1], APVRe, APVIm)
	  V1Im := V1Im + CMultIm(TR2[2], TI2[2], BPVRe, BPVIm)
	  V1Im := V1Im + CMultIm(TR2[3], TI2[3], CPVRe, CPVIm)

	  V1Amount := sqrt((V1Re * V1Re) + (V1Im * V1Im))
      if (V1Im <> 0.0) and (V1Re <> 0.0) then
	    V1Phase := atan2(V1Im, V1Re) * RadToDeg
      endif

	  -- write ("V1Re=", V1Re, "  V1Im=", V1Im, " V1B=", V1Amount)
	  
	  -- positive-sequence system voltage V2
	  V2Re := CMultRe(TR3[1], TI3[1], APVRe, APVIm)
	  V2Re := V2Re + CMultRe(TR3[2], TI3[2], BPVRe, BPVIm)
	  V2Re := V2Re + CMultRe(TR3[3], TI3[3], CPVRe, CPVIm)

	  V2Im := CMultIm(TR3[1], TI3[1], APVRe, APVIm)
	  V2Im := V2Im + CMultIm(TR3[2], TI3[2], BPVRe, BPVIm)
	  V2Im := V2Im + CMultIm(TR3[3], TI3[3], CPVRe, CPVIm)

	  V2Amount := sqrt((V2Re * V2Re) + (V2Im * V2Im))
      if (V1Im <> 0.0) and (V1Re <> 0.0) then
	    V2Phase := atan2(V2Im, V2Re) * RadToDeg
      endif
	  
	  -- zero-sequence system voltage I0	  
	  I0Re := CMultRe(TR1[1], TI1[1], APIRe, APIIm)
	  I0Re := I0Re + CMultRe(TR1[2], TI1[2], BPIRe, BPIIm)
	  I0Re := I0Re + CMultRe(TR1[3], TI1[3], CPIRe, CPIIm)

	  I0Im := CMultIm(TR1[1], TI1[1], APIRe, APIIm)
	  I0Im := I0Im + CMultIm(TR1[2], TI1[2], BPIRe, BPIIm)
	  I0Im := I0Im + CMultIm(TR1[3], TI1[3], CPIRe, CPIIm)

	  I0Amount := sqrt((I0Re * I0Re) + (I0Im * I0Im))
      if (I0Im <> 0.0) and (I0Re <> 0.0) then
        I0Phase := atan2(I0Im, I0Re) * RadToDeg
      endif
	    
      -- write ("-- ++++++++++++++++++++++++++++++++++++++++++++++++++")
	  -- write ("t=", t)
	  -- write ("APVRe=", APVRe, "  APVIm=", APVIm)
	  -- write ("BPVRe=", BPVRe, "  BPVIm=", BPVIm)
	  -- write ("CPVRe=", CPVRe, "  CPVIm=", CPVIm)
	  -- write ("APVB=", APVAmount, "  APVP=", APVPhase)
	  -- write ("BPVB=", BPVAmount, "  BPVP=", BPVPhase)
	  -- write ("CPVB=", CPVAmount, "  CPVP=", CPVPhase)

		-- positive-sequence system voltage I1	     
	  I1Re := CMultRe(TR2[1], TI2[1], APIRe, APIIm)
	  I1Re := I1Re + CMultRe(TR2[2], TI2[2], BPIRe, BPIIm)
	  I1Re := I1Re + CMultRe(TR2[3], TI2[3], CPIRe, CPIIm)

	  I1Im := CMultIm(TR2[1], TI2[1], APIRe, APIIm)
	  I1Im := I1Im + CMultIm(TR2[2], TI2[2], BPIRe, BPIIm)
	  I1Im := I1Im + CMultIm(TR2[3], TI2[3], CPIRe, CPIIm)

	  I1Amount := sqrt((I1Re * I1Re) + (I1Im * I1Im))
      if (I1Im <> 0.0) and (I1Re <> 0.0) then
	    I1Phase := atan2(I1Im, I1Re) * RadToDeg
      endif

	  -- positive-sequence system voltage I2
	  I2Re := CMultRe(TR3[1], TI3[1], APIRe, APIIm)
	  I2Re := I2Re + CMultRe(TR3[2], TI3[2], BPIRe, BPIIm)
	  I2Re := I2Re + CMultRe(TR3[3], TI3[3], CPIRe, CPIIm)

	  I2Im := CMultIm(TR3[1], TI3[1], APIRe, APIIm)
	  I2Im := I2Im + CMultIm(TR3[2], TI3[2], BPIRe, BPIIm)
	  I2Im := I2Im + CMultIm(TR3[3], TI3[3], CPIRe, CPIIm)

	  I2Amount := sqrt((I2Re * I2Re) + (I2Im * I2Im))
      if (I1Im <> 0.0) and (I1Re <> 0.0) then
	    I2Phase := atan2(I2Im, I2Re) * RadToDeg
      endif	
	  
	  -- write ("ATPDES:ATPDesigner specific comment in MODELS")
	  
    ENDEXEC

  ENDMODEL
  -- End of the MODEL implementation
  -- ATPDESIGNER END MODEL SYMM

  -- ATPDESIGNER START BLOCK USE SYMM AS SYMM001
  -- Input Interface at MODELS level: Power Network -> MODELS
  -- Interface from Measuring Probe Prb 001
  INPUT APV001 {TACS(APV001)}
  INPUT BPV001 {TACS(BPV001)}
  INPUT CPV001 {TACS(CPV001)}
  INPUT API001 {TACS(API001)}
  INPUT BPI001 {TACS(BPI001)}
  INPUT CPI001 {TACS(CPI001)}

  -- ATPDESIGNER START USE SYMM001
  -- Start of the MODEL instance (task)
  USE SYMM AS SYMM001

    -- Phase Current A, B, C
    INPUT API := API001
    INPUT BPI := BPI001
    INPUT CPI := CPI001

    INPUT APV := APV001
    INPUT BPV := BPV001
    INPUT CPV := CPV001

  ENDUSE
  -- End of the MODEL instance (task)
  -- ATPDESIGNER END USE SYMM001

  -- ATPDESIGNER START RECORD 
  -- Recording Signals to .PL4-File
  RECORD
    SYMM001.V0Amount AS AV0001
    SYMM001.V1Amount AS AV1001
    SYMM001.V2Amount AS AV2001
    SYMM001.V0Phase AS PV0001
    SYMM001.V1Phase AS PV1001
    SYMM001.V2Phase AS PV2001
    SYMM001.I0Amount AS AI0001
    SYMM001.I1Amount AS AI1001
    SYMM001.I2Amount AS AI2001
    SYMM001.I0Phase AS PI0001
    SYMM001.I1Phase AS PI1001
    SYMM001.I2Phase AS PI2001
  -- ATPDESIGNER END RECORD 
  -- ATPDESIGNER END BLOCK USE SYMM AS SYMM001

ENDMODELS -- End of the MODELS Section
C ATPDESIGNER END BLOCK MODELS
